@@ -18,6 +18,7 @@ import re
 import shlex
 import string
 import subprocess
+import sys
 import time
 import xml
 
@@ -630,6 +631,7 @@ class RobotValidator:
         '^percbotspider',
         '^PercolateCrawler',
         '^perlclient',
+        'PetalBot',
         '^Pingdom',
         '^Pinterest',
         '^PostRank',
@@ -763,6 +765,32 @@ def rate_limit(limit_obj, key, hostname, delta_sec):
     return False
 
 
+# Simple memory cache with expiry mechanism. Stores objects in process so no serialization required, but is not distributed over network.
+# The application must call purge_cache() as needed by the application to expire keys that haven't been accessed.
+class Memcache(dict):
+    def set_cache(self, key, value, expiry_sec):
+        seconds_old = expiry_sec + random.randint(0, expiry_sec)
+        expiry = datetime.datetime.utcnow() + datetime.timedelta(seconds=seconds_old)
+        self[key] = (value, expiry)
+
+    def get_cache(self, key):
+        cache_obj = self.get(key)
+        if cache_obj:
+            value, expiry = cache_obj
+            if expiry > datetime.datetime.utcnow():
+                return value
+
+    def purge_cache(self):
+        utcnow = datetime.datetime.utcnow()
+        delete_keys = []
+        for key, cache_obj in self.items():
+            value, expiry = cache_obj
+            if expiry <= utcnow:
+                delete_keys.append(key)
+        for delete_key in delete_keys:
+            del self[delete_key]
+
+
 class TimerObj(object):
     pass
 
@@ -834,7 +862,7 @@ def incr_start(obj, timer_name):
         setattr(obj, '_timers', {})
     # Do overwrite the previous so we can add up the times
     start_name = '_%s_start_ts' % timer_name
-    obj._timers[start_name] =  time.time()
+    obj._timers[start_name] = time.time()
 
 def incr_stop(obj, timer_name):
     if not obj:
@@ -850,3 +878,31 @@ def incr_stop(obj, timer_name):
     obj._timers[exec_name] += obj._timers[stop_name] - obj._timers[start_name]
     obj._timers[cnt_name] += 1
     return obj._timers[stop_name] - obj._timers[start_name]
+
+
+# Support for MaxMind's free GeoLite2 database, sign up at https://www.maxmind.com/en/geolite2/signup
+# Add geoip2 to setup.py and pip requirements.txt, and set geolite_db_path to the filesystem location of the downloaded db in application config.py
+class GeoLite(object):
+    geolite = None
+    geolite_path = None
+
+    def __init__(self, geolite_path):
+        if not GeoLite.geolite_path:
+            GeoLite.geolite_path = geolite_path
+
+    @classmethod
+    def get_ip(cls, ip_addr):
+        # Only open the database once, and only as-needed
+        if not os.path.exists(cls.geolite_path):
+            logging.error("GeoLite db not found: %s" % cls.geolite_path)
+            return {}
+        import geoip2.database
+        if not cls.geolite:
+            cls.geolite = geoip2.database.Reader(cls.geolite_path)
+        record = cls.geolite.city(ip_addr)
+        geoip = {'continent': record.continent.name, 'country_iso_code': record.country.iso_code, 'country': record.country.name, 'time_zone': record.location.time_zone,
+                 'city': record.city.name, 'ip_addr': ip_addr}
+        if len(record.subdivisions):
+            geoip['region_iso_code'] = record.subdivisions[0].iso_code
+            geoip['region'] = record.subdivisions[0].name
+        return geoip
